@@ -13,12 +13,13 @@
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-#include <config.h>
+#include <sipwitch-config.h>
 #include <ucommon/ucommon.h>
 #include <ucommon/export.h>
 #include <sipwitch/events.h>
 #include <sipwitch/control.h>
 #include <sipwitch/mapped.h>
+#include <sipwitch/service.h>
 
 #if !defined(_MSWINDOWS_)
 #include <sys/un.h>
@@ -127,7 +128,7 @@ void dispatch::send(events *msg)
         next = dp->next;
         if(::send(dp->session, (const char *)msg, sizeof(events), 0) < (ssize_t)sizeof(events)) {
 disconnect:
-            shell::log(DEBUG3, "releasing client events for %d", dp->session);
+            shell::log(DEBUG3, "releasing client events for %ld", (long)dp->session);
             dp->release();
             dp->next = freelist;
             freelist = *dp;
@@ -163,7 +164,8 @@ void event_thread::run(void)
             break;
 
         events::sync();
-        shell::log(DEBUG3, "connecting client events for %d", client);
+        shell::log(DEBUG3, "connecting client events for %ld", (long)client);
+
         msg.type = events::WELCOME;
         msg.server.started = started;
         String::set(msg.server.version, sizeof(msg.server.version), VERSION);
@@ -171,8 +173,12 @@ void event_thread::run(void)
         String::set(msg.server.state, sizeof(msg.server.state), *saved_state);
         String::set(msg.server.realm, sizeof(msg.server.realm), *saved_realm);
         private_locking.release();
-        msg.type = events::WELCOME;
         ::send(client, (const char *)&msg, sizeof(msg), 0);
+
+        String::set(msg.contact, sizeof(msg.contact), *service::getContact());
+        msg.type = events::CONTACT;
+        ::send(client, (const char *)&msg, sizeof(msg), 0);
+
         dispatch::add(client);
     }
 
@@ -197,16 +203,20 @@ bool events::start(void)
 
     memset(&abuf, 0, sizeof(abuf));
 #ifdef  _MSWINDOWS_
+    DWORD port;
+    HKEY keys, subkey;
+    socklen_t alen;
+
     abuf.sin_family = AF_INET;
     abuf.sin_addr.s_addr = inet_addr("127.0.0.1");
     abuf.sin_port = 0;
     if(::bind(ipc, (struct sockaddr *)&abuf, sizeof(abuf)) < 0)
         goto failed;
 
-    socklen_t alen = sizeof(abuf);
+    alen = sizeof(abuf);
     ::getsockname(ipc, (struct sockaddr *)&abuf, &alen);
-    DWORD port = ntohs(abuf.sin_port);
-    HKEY keys = HKEY_LOCAL_MACHINE, subkey;
+    port = ntohs(abuf.sin_port);
+    keys = HKEY_LOCAL_MACHINE;
     if(RegCreateKeyEx(keys, "SOFTWARE\\sipwitch", 0L, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &subkey, NULL) == ERROR_SUCCESS) {
         RegSetValueEx(subkey, "port", 0L, REG_DWORD, (const BYTE *)&port, sizeof(port));
         port = GetCurrentProcessId();
@@ -337,6 +347,33 @@ void events::failure(const char *reason)
     dispatch::send(&msg);
 }
 
+void events::reload(void)
+{
+    events msg;
+
+    msg.type = CONTACT;
+    String::set(msg.contact, sizeof(msg.contact), *service::getContact());
+    dispatch::send(&msg);
+
+    msg.type = PUBLISH;
+    volatile char *addr = service::callback::sip_publish;
+    if(addr) {
+        string_t uri = str("sip:") + (const char *)addr;
+        String::set(msg.contact, sizeof(msg.contact), *uri);
+        dispatch::send(&msg);
+    }
+}
+
+void events::publish(const char *addr)
+{
+    events msg;
+
+    msg.type = PUBLISH;
+    string_t uri = str("sip:") + addr;
+    String::set(msg.contact, sizeof(msg.contact), *uri);
+    dispatch::send(&msg);
+}
+
 void events::terminate(const char *reason)
 {
     events msg;
