@@ -13,7 +13,7 @@
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-#include <config.h>
+#include <sipwitch-config.h>
 #include <ucommon/ucommon.h>
 #include <ucommon/export.h>
 #include <sipwitch/control.h>
@@ -37,9 +37,12 @@ unsigned short service::callback::sip_port = 5060;
 unsigned service::callback::sip_prefix = 0;
 unsigned service::callback::sip_range = 0;
 const char *service::callback::sip_iface = NULL;
+volatile char *service::callback::sip_contact = NULL;
+volatile char *service::callback::sip_publish = NULL;
 int service::callback::sip_protocol = IPPROTO_UDP;
 int service::callback::sip_family = AF_INET;
 int service::callback::sip_tlsmode = 0;
+bool service::callback::sip_public = false;
 const char *service::callback::sip_tlspwd = "";
 const char *service::callback::sip_tlsdev = "/dev/random";
 const char *service::callback::sip_tlsca = "/etc/ssl/ca.pem";
@@ -223,6 +226,9 @@ memalloc(s), root()
 
     root.setId((char *)name);
     root.setPointer(NULL);
+
+    contact = NULL;
+
     if(!started) {
         time(&started);
         time(&periodic);
@@ -248,8 +254,14 @@ void service::publish(const char *addr)
     int i = 0;
     resolver.set(addr, i);
     host = resolver.getAddr();
-    if(host)
+    if(host) {
+        volatile char *old = callback::sip_publish;
+        callback::sip_publish = strdup(addr);
+        if(old)
+            free((char *)old);
         Socket::store(&peering, host);
+        events::publish(addr);
+    }
 }
 
 void service::published(struct sockaddr_storage *peer)
@@ -749,6 +761,42 @@ void service::dumpfile(void)
     fclose(fp);
 }
 
+string_t service::getContact(void)
+{
+    string_t uri;
+    volatile char *vaddr = callback::sip_contact;
+    unsigned short port;
+    const char *addr = (const char *)vaddr;
+
+    if(!addr)
+        addr = getInterface();
+
+    if(!addr || eq(addr, "*")) {
+#ifdef  HAVE_GETHOSTNAME
+        static char hostbuf[256] = {0};
+        gethostname(hostbuf, sizeof(hostbuf));
+        if(hostbuf[0])
+            addr = hostbuf;
+        else
+            addr = "localhost";
+#else
+        addr = "localhost";
+#endif
+    }
+
+    port = getPort();
+    if(port && port != 5060) {
+        if(strchr(addr, ':'))
+            uri = str("sip:[") + addr + "]:" + str(port);
+        else
+            uri = str("sip:") + addr + ":" + str(port);
+    }
+    else
+        uri = str("sip:") + addr;
+
+    return uri;
+}
+
 bool service::period(long slice)
 {
     assert(slice > 0);
@@ -850,6 +898,8 @@ void service::commit(void)
     confirm();
 
     locking.modify();
+    if(contact)
+        callback::sip_contact = (volatile char *)(contact);
     orig = cfg;
     cfg = this;
     locking.commit();
@@ -863,6 +913,9 @@ void service::commit(void)
         }
     }
 
+    // send any config related reload events...
+    events::reload();
+
     // let short-term volatile references settle before we delete it...
     if(orig) {
         Thread::sleep(1000);
