@@ -13,7 +13,7 @@
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-#include <config.h>
+#include <sipwitch-config.h>
 #include <ucommon/ucommon.h>
 #include <ucommon/export.h>
 #include <sipwitch/uri.h>
@@ -88,7 +88,7 @@ bool uri::userid(const char *addr, char *buf, size_t size)
     if(!strchr(addr, '@'))
         return false;
 
-    string::set(buf, size, addr);
+    String::set(buf, size, addr);
     ep = strchr(buf, '@');
     if(ep)
         *ep = 0;
