@@ -180,6 +180,7 @@ public:
     protected:
         friend class service;
         friend class modules;
+        friend class events;
 
         unsigned runlevel;
         bool active_flag;
@@ -188,9 +189,12 @@ public:
         static unsigned count;
         static unsigned short sip_port;
         static const char *sip_iface;
+        static volatile char *sip_contact;
+        static volatile char *sip_publish;
         static int sip_protocol;
         static int sip_family;
         static int sip_tlsmode;
+        static bool sip_public;             // hotspot mode flag
         static const char *sip_domain;
         static const char *sip_realm;
         static const char *sip_tlspwd;
@@ -229,7 +233,10 @@ public:
 
     public:
         inline static void bind(unsigned short port)
-            {sip_port = port;};
+            {sip_port = port;}
+
+        inline static void setPublic(void)
+            {sip_public = true;}
 
         static void bind(const char *addr);
 
@@ -302,6 +309,11 @@ public:
     inline keynode *getRoot(void)
         {return &root;};
 
+    static string_t getContact(void);
+
+    inline void setContact(const char *text)
+        {contact = dup(text);}
+
     static inline const char *getInterface(void)
         {return service::callback::sip_iface;};
 
@@ -331,6 +343,7 @@ protected:
     keynode root;
     stringbuf<1024> buffer;
     LinkedObject *keys[CONFIG_KEY_SIZE];
+    const char *contact;
 
     static service *cfg;
     static condlock_t locking;
