@@ -13,8 +13,8 @@
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+#include <sipwitch-config.h>
 #include <sipwitch/sipwitch.h>
-#include <config.h>
 
 NAMESPACE_SIPWITCH
 using namespace UCOMMON_NAMESPACE;
