@@ -13,12 +13,36 @@
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+#undef  HAVE_CONFIG_H
+
+#include <sipwitch-config.h>
 #include <sipwitch/sipwitch.h>
 #include <ucommon/secure.h>
 #include <eXosip2/eXosip.h>
-#include <config.h>
 #include <ctype.h>
 
+#if defined(EXOSIP_OPT_BASE_OPTION) && !defined(EXOSIP_OPT_DONT_SEND_101)
+#define EXOSIP_API4
+#endif
+
+#ifdef  EXOSIP_API4
+
+typedef eXosip_t *context_t;
+
+#define EXOSIP_CONTEXT  stack::sip.context
+#define OPTION_CONTEXT  stack::sip.context,
+#define EXOSIP_LOCK     eXosip_lock(stack::sip.context);
+#define EXOSIP_UNLOCK   eXosip_unlock(stack::sip.context);
+#else
+
+typedef void *context_t;
+
+#define EXOSIP_CONTEXT
+#define OPTION_CONTEXT
+#define EXOSIP_LOCK     eXosip_lock();
+#define EXOSIP_UNLOCK   eXosip_unlock();
+#endif
+
 #ifndef SESSION_EXPIRES
 #define SESSION_EXPIRES "session-expires"
 #endif
@@ -116,6 +140,7 @@ public:
         } index;
         struct sockaddr_internet address;
         struct sockaddr_storage peering;
+        context_t context;
 
         time_t created;
         status_t status;
@@ -245,6 +270,7 @@ private:
     public:
         registry::mapped *reg;
         int cid, tid, did;
+        context_t context;
         time_t activates;
         uint32_t sequence;
         call *parent;
@@ -385,9 +411,11 @@ private:
     timeout_t ring_timer, cfna_timer, reset_timer;
     unsigned invite_expires;
 
+public:
+    context_t context;
+
     static stack sip;
 
-public:
     class __LOCAL subnet : public cidr
     {
     private:
@@ -597,6 +625,7 @@ private:
     unsigned via_hops;
     unsigned via_port, from_port, contact_port;
     destination_t destination;
+    context_t source_context, target_context;
 
     char *sip_realm;
     osip_proxy_authenticate_t *proxy_auth;
