@@ -44,17 +44,18 @@ static shell::numericopt concurrency('c', "--concurrency", _TEXT("process concur
 static shell::flagopt desktop(0, "--desktop", _TEXT("enable desktop access"));
 static shell::flagopt foreflag('f', "--foreground", _TEXT("run in foreground"));
 #ifdef  HAVE_PWD_H
-static shell::stringopt group('g', "--group", _TEXT("use specified group permissions"), "groupid", "nobody");
+static shell::stringopt group('g', "--group", _TEXT("use specified group permissions"), "groupid", NULL);
 #endif
 static shell::numericopt histbuf('h', "--history", _TEXT("set history buffer"), "count", 0);
 static shell::stringopt loglevel('L', "--logging", _TEXT("set log level"), "level", "err");
 static shell::stringopt loading('l', "--plugins", _TEXT("specify modules to load"), "names", "none");
 static shell::flagopt nolocalusers('n', "--no-localusers", _TEXT("disable local user accounts"));
 static shell::counteropt priority('p', "--priority", _TEXT("set priority level"), "level");
+static shell::flagopt hotspot(0, "--public", _TEXT("public access mode"));
 static shell::flagopt restart('r', "--restartable", _TEXT("set to restartable process"));
 static shell::flagopt trace('t', "--trace", _TEXT("trace sip messages"));
 #ifdef HAVE_PWD_H
-static shell::stringopt user('u', "--user", _TEXT("user to run as"), "userid", "nobody");
+static shell::stringopt user('u', "--user", _TEXT("user to run as"), "userid", NULL);
 #endif
 static shell::flagopt verbose('v', NULL, _TEXT("set verbosity, can be used multiple times"), false);
 static shell::flagopt version(0, "--version", _TEXT("show version information"));
@@ -176,8 +177,18 @@ static bool errlog(shell::loglevel_t level, const char *text)
 
 namespace SIPWITCH_NAMESPACE {
 
-    static void up(void)
+    static void up(const char *pidfile)
     {
+        if(pidfile) {
+            ::remove(pidfile);
+            FILE *fp = fopen(pidfile, "w");
+            if(fp) {
+                fprintf(fp, "    %ld\n", (long)getpid());
+                fclose(fp);
+                fp = NULL;
+            }
+        }
+
         cache::init();
         server::reload();
         server::startup();
@@ -195,6 +206,9 @@ namespace SIPWITCH_NAMESPACE {
         signals::stop();
         service::shutdown();
         control::release();
+
+        if(pidfile)
+            ::remove(pidfile);
     }
 }
 
@@ -213,6 +227,10 @@ static void init(int argc, char **argv, bool detached, shell::mainproc_t svc = N
     corefiles();
     args.getargv0(argv);
 
+    const char *security = args.getenv("SECURITY");
+    if(!security)
+        security = "default";
+
 #if defined(DEBUG)
     if(eq(argv[1], "-gdb") || eq(argv[1], "--gdb") || eq(argv[1], "-dbg") || eq(argv[1], "--dbg")) {
         char *dbg[] = {(char *)"gdb", (char *)"--args", NULL};
@@ -279,6 +297,7 @@ static void init(int argc, char **argv, bool detached, shell::mainproc_t svc = N
     args.setsym("cache", DEFAULT_VARPATH "/cache/sipwitch");
     args.setsym("controls", DEFAULT_VARPATH "/run/sipwitch");
     args.setsym("control", DEFAULT_VARPATH "/run/sipwitch/control");
+    args.setsym("pidfile", DEFAULT_VARPATH "/run/sipwitch/pidfile");
     args.setsym("events", DEFAULT_VARPATH "/run/sipwitch/events");
     args.setsym("config", DEFAULT_CFGPATH "/sipwitch.conf");
     args.setsym("logfiles", DEFAULT_VARPATH "/log");
@@ -325,6 +344,7 @@ static void init(int argc, char **argv, bool detached, shell::mainproc_t svc = N
         args.setsym("controls", rundir);
         args.setsym("control", _STR(str(rundir) + "/control"));
         args.setsym("events", _STR(str(rundir) + "/events"));
+        args.setsym("pidfile", _STR(str(rundir) + "/pidfile"));
         args.setsym("logfiles", rundir);
         args.setsym("siplogs", _STR(str(rundir) + "/siplogs"));
         args.setsym("logfile", _STR(str(rundir) + "/logfile"));
@@ -348,11 +368,11 @@ static void init(int argc, char **argv, bool detached, shell::mainproc_t svc = N
 
 #ifdef  HAVE_PWD_H
     cp = args.getenv("GROUP");
-    if(cp && *cp)
+    if(cp && *cp && !is(group))
         group.set(cp);
 
     cp = args.getenv("USER");
-    if(cp && *cp)
+    if(cp && *cp && !is(user))
         user.set(cp);
 
     // root gets these from default to act as user daemon...
@@ -473,6 +493,9 @@ static void init(int argc, char **argv, bool detached, shell::mainproc_t svc = N
     if(is(debuglevel))
         verbose.set((unsigned)shell::DEBUG0 + *debuglevel);
 
+    if(is(hotspot) || eq(security, "public"))
+        service::callback::setPublic();
+
 #ifdef  HAVE_PWD_H
     pwd = NULL;
     struct group *grp = NULL;
@@ -480,7 +503,7 @@ static void init(int argc, char **argv, bool detached, shell::mainproc_t svc = N
     // if root user, then see if we change permissions...
 
     if(!getuid()) {
-        if(is(user)) {
+        if(*user) {
             if(atoi(*user))
                 pwd = getpwuid(atoi(*user));
             else
@@ -491,7 +514,7 @@ static void init(int argc, char **argv, bool detached, shell::mainproc_t svc = N
         }
     }
 
-    if(is(group)) {
+    if(*group) {
         if(atoi(*group))
             grp = getgrgid(atoi(*group));
         else
@@ -518,8 +541,11 @@ static void init(int argc, char **argv, bool detached, shell::mainproc_t svc = N
     endgrent();
     endpwent();
 
-    if(is(desktop))
+    if(is(desktop) || eq(security, "desktop")) {
         umask(002);
+        service::callback::setPublic();
+    }
+
 #endif
 
     fsys::createDir(rundir, 0775);
@@ -570,7 +596,7 @@ static void init(int argc, char **argv, bool detached, shell::mainproc_t svc = N
     if(is(restart))
         args.restart();
 
-    up();
+    up(args.getsym("pidfile"));
 }
 
 // stub code for windows service daemon...
