@@ -311,9 +311,9 @@ void thread::invite(void)
     msgheader = NULL;
     osip_message_get_subject(sevent->request, 0, &msgheader);
     if(msgheader && msgheader->hvalue && msgheader->hvalue[0])
-        string::set(call->subject, sizeof(call->subject), msgheader->hvalue);
+        String::set(call->subject, sizeof(call->subject), msgheader->hvalue);
     else
-        string::set(call->subject, sizeof(call->subject), "inviting call");
+        String::set(call->subject, sizeof(call->subject), "inviting call");
 
     msgheader = NULL;
     osip_message_get_expires(sevent->request, 0, &msgheader);
@@ -632,6 +632,7 @@ bool thread::authorize(void)
     osip_header_t *msgheader = NULL;
     bool anon = false;
     UserCache *usercache = NULL;
+    bool allowed = false;
 
     if(!sevent->request || !sevent->request->to || !sevent->request->from || !sevent->request->req_uri)
         goto invalid;
@@ -716,15 +717,21 @@ bool thread::authorize(void)
     // bypass request address processing if local domain call....
     memset(&iface, 0, sizeof(iface));
 
-    if(String::equal(registry::getRealm(), uri_host))
+    if(eq(registry::getRealm(), uri_host))
         goto local;
 
-    if(String::equal("localdomain", uri_host))
+    if(eq("localdomain", uri_host))
         goto local;
 
     if(String::ifind(stack::sip.localnames, uri_host, " ,;:\t\n"))
         goto local;
 
+    if(stack::sip_contact && eq((const char *)stack::sip_contact, uri_host))
+        goto local;
+
+    if(stack::sip_publish && eq((const char *)stack::sip_publish, uri_host))
+        goto local;
+
     request_address.set(uri_host, local_port);
 
     if(request_address.getAddr() == NULL) {
@@ -832,6 +839,8 @@ rewrite:
     }
 
 trying:
+    // should SIP_GONE expiration and other checks be here??
+
     if(MSG_IS_PUBLISH(sevent->request))
         return authenticate();
 
@@ -844,9 +853,19 @@ trying:
         return true;
     }
 
-    // see if destination allows anonymous remote (public) callers
+    // see if destination allows anonymous remote (public) callers.
+    // destination must be valid and either have class of service
+    // or server is in hotspot mode.  Origin must offer a host uri.
 
-    if(reginfo && reginfo->isFeature(USER_PROFILE_INCOMING) && from->url->host) {
+    allowed = false;
+    if(reginfo && from && from->url && from->url->host) {
+        if(stack::sip_public)
+            allowed = true;
+        else
+            allowed = reginfo->isFeature(USER_PROFILE_INCOMING);
+    }
+
+    if(allowed) {
 
         // now we make sure the originating call does not "claim" to be
         // associated with us.  If it has any of our our names, then we have
@@ -858,6 +877,12 @@ trying:
         if(String::equal("localdomain", from->url->host))
             return authenticate();
 
+        if(stack::sip_contact && eq((const char *)stack::sip_contact, uri_host))
+            return authenticate();
+
+        if(stack::sip_publish && eq((const char *)stack::sip_publish, uri_host))
+            return authenticate();
+
         if(String::ifind(stack::sip.localnames, from->url->host, " ,;:\t\n"))
             return authenticate();
 
@@ -1012,7 +1037,7 @@ static_routing:
     goto invalid;
 
 anonymous:
-    if(!stack::sip.published) {
+    if(!stack::sip.published && !stack::sip_public) {
         error = SIP_FORBIDDEN;
         goto invalid;
     }
@@ -1028,20 +1053,22 @@ anonymous:
 remote:
     error = SIP_FORBIDDEN;
     destination = EXTERNAL;
-    if(!stack::sip.published)
+    if(!stack::sip.published && !stack::sip_public)
         goto invalid;
 
     if(!authenticate())
         return false;
 
-    // must be active registration to dial out...
+    // must be active registration with correct class of service, or with
+    // server in hotspot mode, to dial out...
     reginfo = registry::access(identity);
     time(&now);
     if(!reginfo || (reginfo->expires && reginfo->expires < now))
         goto invalid;
 
-    if(!reginfo->isFeature(USER_PROFILE_OUTGOING))
-        goto invalid;
+    if(!stack::sip_public)
+        if(!reginfo->isFeature(USER_PROFILE_OUTGOING))
+            goto invalid;
 
     refer = server::referRemote(reginfo, requesting, buffer, sizeof(buffer));
     if(refer)
@@ -1072,35 +1099,35 @@ void thread::send_reply(int error)
 
     osip_message_t *reply = NULL;
 
-    eXosip_lock();
+    EXOSIP_LOCK
     switch(authorizing) {
     case CALL:
-        eXosip_call_build_answer(sevent->tid, error, &reply);
+        eXosip_call_build_answer(OPTION_CONTEXT sevent->tid, error, &reply);
         if(reply != NULL) {
             if(stack::sip_protocol == IPPROTO_UDP)
                 osip_message_set_require(reply, "100rel");
             stack::siplog(reply);
-            eXosip_call_send_answer(sevent->tid, error, reply);
+            eXosip_call_send_answer(OPTION_CONTEXT sevent->tid, error, reply);
         }
         else
-            eXosip_call_send_answer(sevent->tid, SIP_BAD_REQUEST, NULL);
+            eXosip_call_send_answer(OPTION_CONTEXT sevent->tid, SIP_BAD_REQUEST, NULL);
         break;
     case REGISTRAR:
     case MESSAGE:
-        eXosip_message_build_answer(sevent->tid, error, &reply);
+        eXosip_message_build_answer(OPTION_CONTEXT sevent->tid, error, &reply);
         if(reply != NULL) {
             if(stack::sip_protocol == IPPROTO_UDP)
                 osip_message_set_require(reply, "100rel");
             stack::siplog(reply);
-            eXosip_message_send_answer(sevent->tid, error, reply);
+            eXosip_message_send_answer(OPTION_CONTEXT sevent->tid, error, reply);
         }
         else
-            eXosip_call_send_answer(sevent->tid, SIP_BAD_REQUEST, NULL);
+            eXosip_call_send_answer(OPTION_CONTEXT sevent->tid, SIP_BAD_REQUEST, NULL);
         break;
     default:
         break;
     }
-    eXosip_unlock();
+    EXOSIP_UNLOCK
 }
 
 bool thread::authenticate(stack::session *s)
@@ -1148,10 +1175,10 @@ bool thread::authenticate(stack::session *s)
     if(!sip_realm || !*sip_realm || !userid || !secret || !*userid)
         return false;
 
-    eXosip_lock();
-    eXosip_add_authentication_info(userid, userid, secret, NULL, sip_realm);
-    eXosip_automatic_action();
-    eXosip_unlock();
+    EXOSIP_LOCK
+    eXosip_add_authentication_info(OPTION_CONTEXT userid, userid, secret, NULL, sip_realm);
+    eXosip_automatic_action(EXOSIP_CONTEXT);
+    EXOSIP_UNLOCK
     return true;
 }
 
@@ -1276,42 +1303,42 @@ void thread::challenge(void)
         "Digest realm=\"%s\", nonce=\"%s\", algorithm=%s",
                 registry::getRealm(), nonce, registry::getDigest());
 
-    eXosip_lock();
+    EXOSIP_LOCK
     switch(authorizing) {
     case REGISTRAR:
-        eXosip_message_build_answer(sevent->tid, SIP_UNAUTHORIZED, &reply);
+        eXosip_message_build_answer(OPTION_CONTEXT sevent->tid, SIP_UNAUTHORIZED, &reply);
         if(reply != NULL) {
             osip_message_set_header(reply, WWW_AUTHENTICATE, buffer);
             osip_message_set_header(reply, ALLOW, "INVITE, ACK, CANCEL, BYE, REFER, OPTIONS, NOTIFY, SUBSCRIBE, PRACK, MESSAGE, INFO");
             osip_message_set_header(reply, ALLOW_EVENTS , "talk, hold, refer");
             stack::siplog(reply);
-            eXosip_message_send_answer(sevent->tid, SIP_UNAUTHORIZED, reply);
+            eXosip_message_send_answer(OPTION_CONTEXT sevent->tid, SIP_UNAUTHORIZED, reply);
         }
         break;
     case MESSAGE:
-        eXosip_message_build_answer(sevent->tid, SIP_UNAUTHORIZED, &reply);
+        eXosip_message_build_answer(OPTION_CONTEXT sevent->tid, SIP_UNAUTHORIZED, &reply);
         if(reply != NULL) {
             osip_message_set_header(reply, WWW_AUTHENTICATE, buffer);
             osip_message_set_header(reply, ALLOW, "INVITE, ACK, CANCEL, BYE, REFER, OPTIONS, NOTIFY, SUBSCRIBE, PRACK, MESSAGE, INFO");
             osip_message_set_header(reply, ALLOW_EVENTS , "talk, hold, refer");
             stack::siplog(reply);
-            eXosip_message_send_answer(sevent->tid, SIP_UNAUTHORIZED, reply);
+            eXosip_message_send_answer(OPTION_CONTEXT sevent->tid, SIP_UNAUTHORIZED, reply);
         }
         break;
     case CALL:
-        eXosip_call_build_answer(sevent->tid, SIP_UNAUTHORIZED, &reply);
+        eXosip_call_build_answer(OPTION_CONTEXT sevent->tid, SIP_UNAUTHORIZED, &reply);
         if(reply != NULL) {
             osip_message_set_header(reply, ALLOW, "INVITE, ACK, CANCEL, BYE, REFER, OPTIONS, NOTIFY, SUBSCRIBE, PRACK, MESSAGE, INFO");
             osip_message_set_header(reply, ALLOW_EVENTS, "talk, hold, refer");
             osip_message_set_header(reply, WWW_AUTHENTICATE, buffer);
             stack::siplog(reply);
-            eXosip_call_send_answer(sevent->tid, SIP_UNAUTHORIZED, reply);
+            eXosip_call_send_answer(OPTION_CONTEXT sevent->tid, SIP_UNAUTHORIZED, reply);
         }
         break;
     case NONE:
         break;
     }
-    eXosip_unlock();
+    EXOSIP_UNLOCK
 }
 
 bool thread::getsource(void)
@@ -1441,8 +1468,8 @@ reply:
         shell::debug(2, "rejecting %s; error=%d", auth->username, error);
 
     server::release(user);
-    eXosip_lock();
-    eXosip_message_build_answer(sevent->tid, error, &reply);
+    EXOSIP_LOCK
+    eXosip_message_build_answer(OPTION_CONTEXT sevent->tid, error, &reply);
     if(reply != NULL) {
         if(error == SIP_OK) {
             snprintf(temp, sizeof(temp), ";expires=%lu", (long)registry::getExpires());
@@ -1453,11 +1480,11 @@ reply:
         osip_message_set_header(reply, ALLOW, "INVITE, ACK, CANCEL, BYE, REFER, OPTIONS, NOTIFY, SUBSCRIBE, PRACK, MESSAGE, INFO");
         osip_message_set_header(reply, ALLOW_EVENTS , "talk, hold, refer");
         stack::siplog(reply);
-        eXosip_message_send_answer(sevent->tid, error, reply);
+        eXosip_message_send_answer(OPTION_CONTEXT sevent->tid, error, reply);
     }
     else
-        eXosip_message_send_answer(sevent->tid, SIP_BAD_REQUEST, NULL);
-    eXosip_unlock();
+        eXosip_message_send_answer(OPTION_CONTEXT sevent->tid, SIP_BAD_REQUEST, NULL);
+    EXOSIP_UNLOCK
 }
 
 void thread::registration(void)
@@ -1554,8 +1581,8 @@ reply:
             shell::debug(3, "querying %s", reguri->username);
         else
             shell::debug(3, "query rejected for %s; error=%d", reguri->username, error);
-        eXosip_lock();
-        eXosip_message_build_answer(sevent->tid, error, &reply);
+        EXOSIP_LOCK
+        eXosip_message_build_answer(OPTION_CONTEXT sevent->tid, error, &reply);
         if(reply != NULL) {
             if(error == SIP_OK) {
                 snprintf(buftemp, sizeof(buftemp), "<%s:%s@%s>",
@@ -1565,11 +1592,11 @@ reply:
             osip_message_set_header(reply, ALLOW, "INVITE, ACK, CANCEL, BYE, REFER, OPTIONS, NOTIFY, SUBSCRIBE, PRACK, MESSAGE, INFO");
             osip_message_set_header(reply, ALLOW_EVENTS , "talk, hold, refer");
             stack::siplog(reply);
-            eXosip_message_send_answer(sevent->tid, error, reply);
+            eXosip_message_send_answer(OPTION_CONTEXT sevent->tid, error, reply);
         }
         else
-            eXosip_message_send_answer(sevent->tid, SIP_BAD_REQUEST, NULL);
-        eXosip_unlock();
+            eXosip_message_send_answer(OPTION_CONTEXT sevent->tid, SIP_BAD_REQUEST, NULL);
+        EXOSIP_UNLOCK
         return;
     }
 
@@ -1656,8 +1683,8 @@ void thread::reregister(const char *contact, time_t interval)
         }
     }
 reply:
-    eXosip_lock();
-    eXosip_message_build_answer(sevent->tid, answer, &reply);
+    EXOSIP_LOCK
+    eXosip_message_build_answer(OPTION_CONTEXT sevent->tid, answer, &reply);
     if(reply != NULL) {
 
         if(answer == SIP_OK) {
@@ -1674,11 +1701,11 @@ reply:
         osip_message_set_header(reply, ALLOW, "INVITE, ACK, CANCEL, BYE, REFER, OPTIONS, NOTIFY, SUBSCRIBE, PRACK, MESSAGE, INFO");
         osip_message_set_header(reply, ALLOW_EVENTS, "talk, hold, refer");
         stack::siplog(reply);
-        eXosip_message_send_answer(sevent->tid, answer, reply);
+        eXosip_message_send_answer(OPTION_CONTEXT sevent->tid, answer, reply);
     }
     else
-        eXosip_message_send_answer(sevent->tid, SIP_BAD_REQUEST, NULL);
-    eXosip_unlock();
+        eXosip_message_send_answer(OPTION_CONTEXT sevent->tid, SIP_BAD_REQUEST, NULL);
+    EXOSIP_UNLOCK
     if(reginfo && reginfo->isUser() && answer == SIP_OK)
         messages::update(identity);
 }
@@ -1719,17 +1746,17 @@ void thread::options(void)
 {
     osip_message_t *reply = NULL;
 
-    eXosip_lock();
-    if(eXosip_options_build_answer(sevent->tid, SIP_OK, &reply) == 0) {
+    EXOSIP_LOCK
+    if(eXosip_options_build_answer(OPTION_CONTEXT sevent->tid, SIP_OK, &reply) == 0) {
         osip_message_set_header(reply, ACCEPT, "application/sdp, text/plain");
         osip_message_set_header(reply, ALLOW, "INVITE,ACK,CANCEL,OPTIONS,INFO,REFER,MESSAGE,SUBSCRIBE,NOTIFY,REGISTER,PRACK");
         osip_message_set_header(reply, ALLOW_EVENTS, "talk, hold, refer");
         stack::siplog(reply);
-        eXosip_options_send_answer(sevent->tid, SIP_OK, reply);
+        eXosip_options_send_answer(OPTION_CONTEXT sevent->tid, SIP_OK, reply);
     }
     else
-        eXosip_options_send_answer(sevent->tid, SIP_BAD_REQUEST, NULL);
-    eXosip_unlock();
+        eXosip_options_send_answer(OPTION_CONTEXT sevent->tid, SIP_BAD_REQUEST, NULL);
+    EXOSIP_UNLOCK
 }
 
 void thread::shutdown(void)
@@ -1737,7 +1764,7 @@ void thread::shutdown(void)
     shutdown_flag = true;
     while(active_count)
         Thread::sleep(50);
-    eXosip_quit();
+    eXosip_quit(EXOSIP_CONTEXT);
     while(shutdown_count < startup_count)
         Thread::sleep(50);
 }
@@ -1790,7 +1817,7 @@ void thread::run(void)
         identbuf[0] = 0;
 
         if(!shutdown_flag)
-            sevent = eXosip_event_wait(0, stack::sip.timing);
+            sevent = eXosip_event_wait(OPTION_CONTEXT 0, stack::sip.timing);
 
         activated = false;
         accepted = NULL;
