@@ -13,9 +13,9 @@
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+#include <sipwitch-config.h>
 #include <sipwitch/sipwitch.h>
 #include <ctype.h>
-#include <config.h>
 #include <sys/stat.h>
 
 using namespace UCOMMON_NAMESPACE;
