@@ -13,8 +13,9 @@
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-#include "sipwitch/sipwitch.h"
-#include "ucommon/secure.h"
+#include <sipwitch-config.h>
+#include <sipwitch/sipwitch.h>
+#include <ucommon/secure.h>
 #ifdef _MSWINDOWS_
 #include <windows.h>
 #include <io.h>
@@ -24,7 +25,6 @@
 #include <grp.h>
 #include <fcntl.h>
 #endif
-#include <config.h>
 
 #if !defined(_MSWINDOWS_)
 #include <sys/un.h>
@@ -59,9 +59,9 @@ static char *getpass(const char *prompt)
 }
 #endif
 
+#ifndef _MSWINDOWS_
 static void capture(void)
 {
-#ifndef _MSWINDOWS_
     char buffer[512];
     FILE *fp;
 
@@ -72,8 +72,8 @@ static void capture(void)
         fputs(buffer, stdout);
     if(fp)
         fclose(fp);
-#endif
 }
+#endif
 
 static void paddress(struct sockaddr_internet *a1, struct sockaddr_internet *a2)
 {
@@ -426,6 +426,9 @@ static void showevents(char **argv)
     struct passwd *pwd = getpwuid(getuid());
 #endif
 
+    static string_t contact = "-";
+    static string_t publish = "-";
+
     if(argv[1])
         shell::errexit(1, "*** sipwitch: events: no arguments used\n");
 
@@ -435,15 +438,13 @@ static void showevents(char **argv)
     memset(&addr, 0, sizeof(addr));
 
 #ifdef  _MSWINDOWS_
-    DWORD port = 0, pid = 0;
-    DWORD plen;
+    DWORD port = 0;
     DWORD index = 0;
     TCHAR keyname[128];
     TCHAR keyvalue[128];
     DWORD size = sizeof(keyname), vsize = sizeof(keyvalue), vtype;
     DWORD *dp;
 
-    plen = sizeof(port);
     HKEY keys = HKEY_LOCAL_MACHINE, subkey;
     if(RegOpenKeyEx(keys, "SOFTWARE\\sipwitch", 0, KEY_READ, &subkey) != ERROR_SUCCESS)
         shell::errexit(10, "*** sipwitch: events: no service found\n");
@@ -451,8 +452,8 @@ static void showevents(char **argv)
         dp = (DWORD *)&keyvalue;
         if(eq("port", keyname))
             port = *dp;
-        else if(eq("pid", keyname))
-            pid = *dp;
+//      else if(eq("pid", keyname))
+//          pid = *dp;
         vsize = sizeof(keyvalue);
         size = sizeof(keyname);
     }
@@ -491,6 +492,18 @@ static void showevents(char **argv)
         case events::NOTICE:
             printf("notice:  %s\n", event.reason);
             break;
+        case events::CONTACT:
+            if(!eq(contact, event.contact)) {
+                printf("contact: %s\n", event.contact);
+                contact ^= event.contact;
+            }
+            break;
+        case events::PUBLISH:
+            if(!eq(publish, event.contact)) {
+                printf("publish: %s\n", event.contact);
+                publish ^= event.contact;
+            }
+            break;
         case events::WELCOME:
             printf("server version %s %s\n",
                 event.server.version, event.server.state);
@@ -760,6 +773,7 @@ static void usage(void)
         "  calls                    List active calls on server\n"
         "  check                    Server deadlock check\n"
         "  concurrency <level>      Server concurrency level\n"
+        "  contact                  Server contact config address\n"
         "  digest id [realm [type]] Compute a digest\n"
         "  disable conf-id...       Disable configurations\n"
         "  down                     Shut down server\n"
@@ -931,15 +945,17 @@ static void grant(char **argv)
     else if(atol(argv[1]))
         gid = atol(argv[1]);
     else
-        shell::errexit(2, "*** sipwitch: grant: %s: unknown group\n", argv[1]);
+        shell::errexit(2, "*** sipwitch: grant: %s: unknown group", argv[1]);
 
     fsys::fileinfo(DEFAULT_VARPATH "/lib/sipwitch", &ino);
     chmod(DEFAULT_VARPATH "/lib/sipwitch", ino.st_mode | 070);
-    chown(DEFAULT_VARPATH "/lib/sipwitch", ino.st_uid, gid);
+    if(chown(DEFAULT_VARPATH "/lib/sipwitch", ino.st_uid, gid))
+        shell::errexit(2, "*** sipwitch: grant: %s: cannot change owner", argv[1]);
 
     fsys::fileinfo(DEFAULT_VARPATH "/cache/sipwitch", &ino);
     chmod(DEFAULT_VARPATH "/cache/sipwitch", ino.st_mode | 070);
-    chown(DEFAULT_VARPATH "/cache/sipwitch", ino.st_uid, gid);
+    if(chown(DEFAULT_VARPATH "/cache/sipwitch", ino.st_uid, gid))
+        shell::errexit(2, "*** sipwitch: grant: %s: cannot change owner", argv[1]);
 
     exit(0);
 }
@@ -1005,7 +1021,7 @@ PROGRAM_MAIN(argc, argv)
         version();
     else if(eq(*argv, "help") || eq(*argv, "-help") || eq(*argv, "--help"))
         usage();
-    else if(eq(*argv, "reload") || eq(*argv, "check") || eq(*argv, "snapshot") || eq(*argv, "dump") || eq(*argv, "siplog") || eq(*argv, "usercache") || eq(*argv, "policy"))
+    else if(eq(*argv, "reload") || eq(*argv, "check") || eq(*argv, "snapshot") || eq(*argv, "dump") || eq(*argv, "siplog") || eq(*argv, "usercache") || eq(*argv, "policy") || eq(*argv, "contact"))
         single(argv, 30);
     else if(eq(*argv, "history")) {
         if(argc == 2)
